(function() {
  var $input, $talk, clear, createPlayer, drawPlayer, hideTalk, initialize, installHandlers, me, mePlayer, removePlayer, showTalk, submitTalk, sync;
  sync = new ObjectSync();
  me = null;
  $talk = $('#talk');
  $input = $talk.find('input');
  mePlayer = function() {
    return $(".player_" + me).data('player');
  };
  showTalk = function() {
    if (!$talk.is(':visible')) {
      $talk.show();
      return $input.focus();
    }
  };
  submitTalk = function() {
    var new_say, player;
    if (!$talk.is(':visible')) {
      return;
    }
    new_say = $('#talk input').val();
    if (new_say === '') {
      return;
    }
    hideTalk();
    player = mePlayer();
    player.says = new_say;
    return sync.save(player, function(err) {
      if (err) {
        return console.error(err);
      }
    });
  };
  hideTalk = function() {
    $talk.fadeOut('fast');
    return $input.val('').blur();
  };
  installHandlers = function() {
    var arrow;
    arrow = {
      left: 37,
      up: 38,
      right: 39,
      down: 40
    };
    $(window).keydown(function(e) {
      var key, player;
      key = e.keyCode || e.which;
      player = mePlayer();
      switch (key) {
        case arrow.up:
          player.y -= 10;
          return sync.save(player);
        case arrow.right:
          player.x += 10;
          return sync.save(player);
        case arrow.down:
          player.y += 10;
          return sync.save(player);
        case arrow.left:
          player.x -= 10;
          return sync.save(player);
        default:
          return showTalk();
      }
    });
    return $(window).keyup(function(e) {
      var key;
      key = e.keyCode || e.which;
      switch (key) {
        case arrow.left:
        case arrow.right:
        case arrow.up:
        case arrow.down:
          return;
        case 13:
          return submitTalk();
        case 27:
          return hideTalk();
      }
    });
  };
  clear = function() {
    $('.player').remove();
    return me = null;
  };
  drawPlayer = function(player) {
    var $player, $says, old_says;
    $player = $(".player_" + player.id);
    if (!$player.length) {
      $player = $('<div>').addClass('player').addClass("player_" + player.id).append($('<div class="says"></div>')).append($('<div class="avatar avatar_' + player.avatar_type + '"></div>')).appendTo('body');
      if (player.id === me) {
        $player.addClass('me');
      }
    }
    $player.data('player', player);
    $player.css({
      top: player.y,
      left: player.x
    });
    $says = $player.find('.says');
    old_says = $says.text();
    if (player.says !== old_says) {
      return $says.hide().fadeIn().text(player.says);
    }
  };
  removePlayer = function(id) {
    return $(".player_" + id).fadeOut(function() {
      return $(this).remove();
    });
  };
  initialize = function() {
    sync.connect();
    return $.getJSON('/init', function(result) {
      return sync.fetch(result, function(err, objects) {
        var player, _i, _len, _results;
        if (err) {
          console.error(err);
          return alert('boo, that didnt work');
        }
        _results = [];
        for (_i = 0, _len = objects.length; _i < _len; _i++) {
          player = objects[_i];
          _results.push(drawPlayer(player));
        }
        return _results;
      });
    });
  };
  createPlayer = function() {
    var player;
    player = {
      x: Math.floor(Math.random() * 400) + 11,
      y: Math.floor(Math.random() * 300) + 71,
      says: "I'm new here, talk to me",
      avatar_type: Math.floor(Math.random() * 5) + 1
    };
    return sync.save(player, function(err, player) {
      if (err) {
        console.error(err);
        return alert('Awwwww. no. *dies*');
      }
      return me = player.id;
    });
  };
  $(function() {
    sync.connect();
    sync.on('create', function(obj) {
      return drawPlayer(obj);
    });
    sync.on('update', function(obj) {
      return drawPlayer(obj);
    });
    sync.on('destroy', function(id) {
      return removePlayer(id);
    });
    sync.on('connect', function() {
      clear();
      initialize();
      installHandlers();
      return createPlayer();
    });
    return window.sync = sync;
  });
}).call(this);
